-- inprod.lua
-- (C) 2013 Lesley De Cruz
-- See LICENSE.txt for license information.

------------------------------------------------------------------------
-- Inner products between the truncated set of basis functions for the
-- ocean and atmosphere streamfunction fields.
------------------------------------------------------------------------

local params = require("params")
local n, alpha = params.m.n, params.m.alpha
local pi, exp, sqrt = math.pi, math.exp, math.sqrt

-- Convert sparse arrays stored as {coordinate1, coordinate2, ..., value} to sparse tables:
local ctt = require("tensor").coo_to_sparse_table

--- Inner products in the equations for the ocean
local ocean = {}

-- Nonlinear coefficients:

--- `C_{i,j,k} = (\phi_i, J(\phi_j,\nabla^2 \phi_k))_o`.
ocean.C = ctt({
  {1, 1, 2,  (-2*(1 + exp((2*pi*alpha)/n))*n^4*alpha*(48 + 3*n^2 - 4*alpha^2))/(exp((2*pi*alpha)/n)*pi*(9*n^4 + 40*n^2*alpha^2 + 16*alpha^4))},
  {1, 2, 1,  ((1 + exp((2*pi*alpha)/n))*n^4*alpha*(24 + 15*n^2 - 4*alpha^2))/(exp((2*pi*alpha)/n)*pi*(9*n^4 + 40*n^2*alpha^2 + 16*alpha^4))},
  {1, 1, 4,  ((-1 + exp((2*pi*alpha)/n))*n^4*alpha*(-12 + n^2 + alpha^2))/(exp((2*pi*alpha)/n)*pi*(4*n^4 + 5*n^2*alpha^2 + alpha^4))},
  {1, 4, 1,  ((-1 + exp((2*pi*alpha)/n))*n^4*alpha*(12 + 19*n^2 - 2*alpha^2))/(4.*exp((2*pi*alpha)/n)*pi*(4*n^4 + 5*n^2*alpha^2 + alpha^4))},
  {1, 2, 3,  ((-1 + exp((2*pi*alpha)/n))*n^4*(3*(n^2 + n^4) + 2*(3 + n^2)*alpha^2 - alpha^4))/(2.*exp((2*pi*alpha)/n)*pi*alpha*(n^2 + alpha^2)*(4*n^2 + alpha^2))},
  {1, 3, 2,  -((-1 + exp((2*pi*alpha)/n))*n^4*(3*n^2*(16 + n^2) + 16*(6 + n^2)*alpha^2 - 8*alpha^4))/(8.*exp((2*pi*alpha)/n)*pi*alpha*(n^2 + alpha^2)*(4*n^2 + alpha^2))},
  {1, 3, 4,  (-8*(1 + exp((2*pi*alpha)/n))*n^4*alpha*(39*n^4 - 16*n^2*(-21 + alpha^2) - 16*alpha^2*(-12 + alpha^2)))/
             (exp((2*pi*alpha)/n)*pi*(n^2 + 4*alpha^2)*(9*n^2 + 4*alpha^2)*(25*n^2 + 4*alpha^2))},
  {1, 4, 3,  (4*(1 + exp((2*pi*alpha)/n))*n^4*alpha*(303*n^4 - 16*alpha^2*(-6 + alpha^2) + 8*n^2*(21 + 13*alpha^2)))/
             (exp((2*pi*alpha)/n)*pi*(n^2 + 4*alpha^2)*(9*n^2 + 4*alpha^2)*(25*n^2 + 4*alpha^2))},
  {2, 3, 1,  ((-1 + exp((2*pi*alpha)/n))*n^4*(3*n^4 - 4*alpha^4 + n^2*(12 - 22*alpha^2)))/(8.*exp((2*pi*alpha)/n)*pi*(4*n^4*alpha + 5*n^2*alpha^3 + alpha^5))},
  {2, 1, 3,  -((-1 + exp((2*pi*alpha)/n))*n^4*(3*n^4 + alpha^4 + n^2*(3 + 4*alpha^2)))/(2.*exp((2*pi*alpha)/n)*pi*(4*n^4*alpha + 5*n^2*alpha^3 + alpha^5))},
  {2, 1, 1,  -(((1 + exp((-2*pi*alpha)/n))*n^4*alpha)/(pi*(n^2 + 4*alpha^2)))},
  {2, 3, 3,  (-4*(1 + exp((-2*pi*alpha)/n))*n^4*alpha)/(pi*(n^2 + 4*alpha^2))},
  {3, 1, 2,  ((-1 + exp((2*pi*alpha)/n))*n^4*(3*n^2*(16 + n^2) - 4*(24 + 7*n^2)*alpha^2 + 8*alpha^4))/(8.*exp((2*pi*alpha)/n)*pi*alpha*(n^2 + alpha^2)*(4*n^2 + alpha^2))},
  {3, 2, 1,  -((-1 + exp((2*pi*alpha)/n))*n^4*(3*n^2*(4 + n^2) - 4*(6 + 11*n^2)*alpha^2 + 4*alpha^4))/(8.*exp((2*pi*alpha)/n)*pi*alpha*(n^2 + alpha^2)*(4*n^2 + alpha^2))},
  {3, 2, 3,  (4*(1 + exp((2*pi*alpha)/n))*n^4*alpha*(345*n^4 - 16*alpha^2*(-6 + alpha^2) + 8*n^2*(15 + 16*alpha^2)))/
             (exp((2*pi*alpha)/n)*pi*(225*n^6 + 1036*n^4*alpha^2 + 560*n^2*alpha^4 + 64*alpha^6))},
  {3, 3, 2,  (-8*(1 + exp((2*pi*alpha)/n))*n^4*alpha*(15*n^4 - 16*alpha^2*(-12 + alpha^2) + 8*n^2*(30 + 7*alpha^2)))/
             (exp((2*pi*alpha)/n)*pi*(225*n^6 + 1036*n^4*alpha^2 + 560*n^2*alpha^4 + 64*alpha^6))},
  {3, 1, 4,  (-8*(1 + exp((2*pi*alpha)/n))*n^4*alpha*(81*n^4 - 16*alpha^2*(-12 + alpha^2) + 8*n^2*(18 + alpha^2)))/
             (exp((2*pi*alpha)/n)*pi*(n^2 + 4*alpha^2)*(9*n^2 + 4*alpha^2)*(25*n^2 + 4*alpha^2))},
  {3, 4, 1,  (4*(1 + exp((2*pi*alpha)/n))*n^4*alpha*(63*n^4 - 16*alpha^2*(-6 + alpha^2) + 8*n^2*(9 + 31*alpha^2)))/
             (exp((2*pi*alpha)/n)*pi*(n^2 + 4*alpha^2)*(9*n^2 + 4*alpha^2)*(25*n^2 + 4*alpha^2))},
  {3, 3, 4,  (-4*(-1 + exp((2*pi*alpha)/n))*n^4*alpha*(12 + 3*n^2 - alpha^2))/(exp((2*pi*alpha)/n)*pi*(9*n^4 + 10*n^2*alpha^2 + alpha^4))},
  {3, 4, 3,  (2*(-1 + exp((2*pi*alpha)/n))*n^4*alpha*(6 + 15*n^2 - alpha^2))/(exp((2*pi*alpha)/n)*pi*(9*n^4 + 10*n^2*alpha^2 + alpha^4))},
  {4, 3, 3,  (-2*(1 - exp((-2*pi*alpha)/n))*n^4*alpha)/(pi*(n^2 + alpha^2))},
  {4, 1, 1,  -((1 - exp((-2*pi*alpha)/n))*n^4*alpha)/(2.*pi*(n^2 + alpha^2))},
  {4, 3, 1,  (-4*(1 + exp((2*pi*alpha)/n))*n^4*alpha*(33*n^4 + 16*alpha^4 + 8*n^2*(-6 + 17*alpha^2)))/
             (exp((2*pi*alpha)/n)*pi*(225*n^6 + 1036*n^4*alpha^2 + 560*n^2*alpha^4 + 64*alpha^6))},
  {4, 1, 3,  (-4*(1 + exp((2*pi*alpha)/n))*n^4*alpha*(183*n^4 + 16*alpha^4 + 16*n^2*(3 + 7*alpha^2)))/
             (exp((2*pi*alpha)/n)*pi*(225*n^6 + 1036*n^4*alpha^2 + 560*n^2*alpha^4 + 64*alpha^6))},
})
-- Linear coefficients:

--- `K_{i,j} = (\phi_i, \nabla^2 F_j)_o`.
-- Forcing of the atmosphere on the ocean.
ocean.K = ctt({
  {1, 2,      (2*(1 + exp((2*pi*alpha)/n))*n^2*(1 + n^2)*(3*n^2 - 4*alpha^2))/(pi*(9*n^4 + 40*n^2*alpha^2 + 16*alpha^4))},
  {1, 3,      (16*(1 + exp((2*pi*alpha)/n))*n^3*(1 + n^2)*alpha)/(pi*(9*n^4 + 40*n^2*alpha^2 + 16*alpha^4))},
  {1, 4,      (16*sqrt(2)*(1 + exp((2*pi*alpha)/n))*n^2)/(3.*pi^2*(n^2 + 4*alpha^2))},
  {2, 1,      (-8*sqrt(2)*(1 + exp((2*pi*alpha)/n))*n^2)/(3.*pi^2*(n^2 + 4*alpha^2))},
  {2, 5,      (2*(1 + exp((2*pi*alpha)/n))*n^2*(4 + n^2)*(3*n^2 - 4*alpha^2))/(pi*(9*n^4 + 40*n^2*alpha^2 + 16*alpha^4))},
  {2, 6,      (16*(1 + exp((2*pi*alpha)/n))*n^3*(4 + n^2)*alpha)/(pi*(9*n^4 + 40*n^2*alpha^2 + 16*alpha^4))},
  {3, 2,      ((-1 + exp((2*pi*alpha)/n))*n^2*(1 + n^2))/(pi*(4*n^2 + alpha^2))},
  {3, 3,      (-2*(-1 + exp((2*pi*alpha)/n))*n^3*(1 + n^2))/(pi*(4*n^2*alpha + alpha^3))},
  {3, 4,      (-8*sqrt(2)*(-1 + exp((2*pi*alpha)/n))*n^2)/(3.*pi^2*(n^2 + alpha^2))},
  {4, 1,      (4*sqrt(2)*(-1 + exp((2*pi*alpha)/n))*n^2)/(3.*pi^2*(n^2 + alpha^2))},
  {4, 5,      ((-1 + exp((2*pi*alpha)/n))*n^2*(4 + n^2))/(pi*(4*n^2 + alpha^2))},
  {4, 6,      (-2*(-1 + exp((2*pi*alpha)/n))*n^3*(4 + n^2))/(pi*(4*n^2*alpha + alpha^3))},
  {1, 7,      (2*(1 + exp((2*pi*alpha)/n))*n^2*(1 + 4*n^2)*(15*n^2 - 4*alpha^2))/(pi*(225*n^4 + 136*n^2*alpha^2 + 16*alpha^4))},
  {1, 8,      (32*(1 + exp((2*pi*alpha)/n))*n^3*(1 + 4*n^2)*alpha)/(pi*(225*n^4 + 136*n^2*alpha^2 + 16*alpha^4))},
  {2, 9,      (8*(1 + exp((2*pi*alpha)/n))*n^2*(1 + n^2)*(15*n^2 - 4*alpha^2))/(pi*(225*n^4 + 136*n^2*alpha^2 + 16*alpha^4))},
  {2, 10,      (128*(1 + exp((2*pi*alpha)/n))*n^3*(1 + n^2)*alpha)/(pi*(225*n^4 + 136*n^2*alpha^2 + 16*alpha^4))},
  {3, 7,      -(((-1 + exp((2*pi*alpha)/n))*n^2*(1 + 4*n^2)*(3*n^2 - alpha^2))/(pi*(9*n^4 + 10*n^2*alpha^2 + alpha^4)))},
  {3, 8,      (-4*(-1 + exp((2*pi*alpha)/n))*n^3*(1 + 4*n^2)*alpha)/(pi*(9*n^4 + 10*n^2*alpha^2 + alpha^4))},
  {4, 9,      (-4*(-1 + exp((2*pi*alpha)/n))*n^2*(1 + n^2)*(3*n^2 - alpha^2))/(pi*(9*n^4 + 10*n^2*alpha^2 + alpha^4))},
  {4, 10,      (-16*(-1 + exp((2*pi*alpha)/n))*n^3*(1 + n^2)*alpha)/(pi*(9*n^4 + 10*n^2*alpha^2 + alpha^4))}
})
--- `M_{i,j} = (phi_i, \nabla^2 \phi_j)_o`.
-- Forcing of the ocean fields on the ocean.
ocean.M = ctt({
  {1, 1,      (-4 - n^2 + 4*alpha^2)/4},
  {1, 3,      (8*n*alpha)/(3*pi)},
  {2, 2,      (-16 - n^2 + 4*alpha^2)/4},
  {2, 4,      (8*n*alpha)/(3*pi)},
  {3, 1,      (-8*n*alpha)/(3*pi)},
  {3, 3,      -1 - n^2 + alpha^2},
  {4, 2,      (-8*n*alpha)/(3*pi)},
  {4, 4,      -4 - n^2 + alpha^2},
})
--- `N_{i,j} = (phi_i, \partial_x \phi_j)_o`.
-- Beta term
ocean.N = ctt({
  {1, 1,      -alpha},
  {1, 3,      (-4*n)/(3*pi)},
  {2, 2,      -alpha},
  {2, 4,      (-4*n)/(3*pi)},
  {3, 1,      (4*n)/(3*pi)},
  {3, 3,      -alpha},
  {4, 2,      (4*n)/(3*pi)},
  {4, 4,      -alpha},
})

--- Inner products in the equations for the atmosphere
local atmos = {}
--- `a_{i,j} = (F_i, \nabla^2 F_j)_a`.
atmos.a = ctt({
  {1, 1,      -1},
  {2, 2,      -1 - n^2},
  {3, 3,      -1 - n^2},
  {4, 4,      -4},
  {5, 5,      -4 - n^2},
  {6, 6,      -4 - n^2},
  {7, 7,      -1 - 4*n^2},
  {8, 8,      -1 - 4*n^2},
  {9, 9,      -4*(1 + n^2)},
  {10, 10,    -4*(1 + n^2)},
})
--- `b_{i,j} = (F_i, J(F_j, \nabla^2 F_k))_a`.
atmos.b = ctt({
  {1, 2, 3,  (8*sqrt(2)*n*(1 + n^2))/(3*pi)},
  {1, 3, 2,  (-8*sqrt(2)*n*(1 + n^2))/(3*pi)},
  {1, 5, 6,  (32*sqrt(2)*n*(4 + n^2))/(15*pi)},
  {1, 6, 5,  (-32*sqrt(2)*n*(4 + n^2))/(15*pi)},
  {1, 7, 8,  (16*sqrt(2)*n*(1 + 4*n^2))/(3*pi)},
  {1, 8, 7,  (-16*sqrt(2)*n*(1 + 4*n^2))/(3*pi)},
  {1, 9, 10, (256*sqrt(2)*n*(1 + n^2))/(15*pi)},
  {1, 10, 9, (-256*sqrt(2)*n*(1 + n^2))/(15*pi)},
  {2, 1, 3,  (-8*sqrt(2)*n*(1 + n^2))/(3*pi)},
  {2, 3, 1,  (8*sqrt(2)*n)/(3*pi)},
  {2, 4, 6,  (-64*sqrt(2)*n*(4 + n^2))/(15*pi)},
  {2, 5, 8,  (-3*n*(1 + 4*n^2))/2},
  {2, 6, 4,  (256*sqrt(2)*n)/(15*pi)},
  {2, 6, 7,  (3*(n + 4*n^3))/2},
  {2, 7, 6,  (-3*n*(4 + n^2))/2},
  {2, 8, 5,  (3*n*(4 + n^2))/2},
  {3, 1, 2,  (8*sqrt(2)*n*(1 + n^2))/(3*pi)},
  {3, 2, 1,  (-8*sqrt(2)*n)/(3*pi)},
  {3, 4, 5,  (64*sqrt(2)*n*(4 + n^2))/(15*pi)},
  {3, 5, 4,  (-256*sqrt(2)*n)/(15*pi)},
  {3, 5, 7,  (3*(n + 4*n^3))/2},
  {3, 6, 8,  (3*(n + 4*n^3))/2},
  {3, 7, 5,  (-3*n*(4 + n^2))/2},
  {3, 8, 6,  (-3*n*(4 + n^2))/2},
  {4, 2, 6,  (64*sqrt(2)*n*(4 + n^2))/(15*pi)},
  {4, 3, 5,  (-64*sqrt(2)*n*(4 + n^2))/(15*pi)},
  {4, 5, 3,  (64*sqrt(2)*n*(1 + n^2))/(15*pi)},
  {4, 6, 2,  (-64*sqrt(2)*n*(1 + n^2))/(15*pi)},
  {4, 7, 10, (512*sqrt(2)*n*(1 + n^2))/(15*pi)},
  {4, 8, 9,  (-512*sqrt(2)*n*(1 + n^2))/(15*pi)},
  {4, 9, 8,  (128*sqrt(2)*n*(1 + 4*n^2))/(15*pi)},
  {4, 10, 7, (-128*sqrt(2)*n*(1 + 4*n^2))/(15*pi)},
  {5, 1, 6,  (-32*sqrt(2)*n*(4 + n^2))/(15*pi)},
  {5, 2, 8,  (3*(n + 4*n^3))/2},
  {5, 3, 4,  (256*sqrt(2)*n)/(15*pi)},
  {5, 3, 7,  (-3*n*(1 + 4*n^2))/2},
  {5, 4, 3,  (-64*sqrt(2)*n*(1 + n^2))/(15*pi)},
  {5, 6, 1,  (32*sqrt(2)*n)/(15*pi)},
  {5, 7, 3,  (3*n*(1 + n^2))/2},
  {5, 8, 2,  (-3*n*(1 + n^2))/2},
  {6, 1, 5,  (32*sqrt(2)*n*(4 + n^2))/(15*pi)},
  {6, 2, 4,  (-256*sqrt(2)*n)/(15*pi)},
  {6, 2, 7,  (-3*n*(1 + 4*n^2))/2},
  {6, 3, 8,  (-3*n*(1 + 4*n^2))/2},
  {6, 4, 2,  (64*sqrt(2)*n*(1 + n^2))/(15*pi)},
  {6, 5, 1,  (-32*sqrt(2)*n)/(15*pi)},
  {6, 7, 2,  (3*n*(1 + n^2))/2},
  {6, 8, 3,  (3*n*(1 + n^2))/2},
  {7, 1, 8,  (-16*sqrt(2)*n*(1 + 4*n^2))/(3*pi)},
  {7, 2, 6,  (3*n*(4 + n^2))/2},
  {7, 3, 5,  (3*n*(4 + n^2))/2},
  {7, 4, 10, (-512*sqrt(2)*n*(1 + n^2))/(15*pi)},
  {7, 5, 3,  (-3*n*(1 + n^2))/2},
  {7, 6, 2,  (-3*n*(1 + n^2))/2},
  {7, 8, 1,  (16*sqrt(2)*n)/(3*pi)},
  {7, 10, 4, (512*sqrt(2)*n)/(15*pi)},
  {8, 1, 7,  (16*sqrt(2)*n*(1 + 4*n^2))/(3*pi)},
  {8, 2, 5,  (-3*n*(4 + n^2))/2},
  {8, 3, 6,  (3*n*(4 + n^2))/2},
  {8, 4, 9,  (512*sqrt(2)*n*(1 + n^2))/(15*pi)},
  {8, 5, 2,  (3*n*(1 + n^2))/2},
  {8, 6, 3,  (-3*n*(1 + n^2))/2},
  {8, 7, 1,  (-16*sqrt(2)*n)/(3*pi)},
  {8, 9, 4,  (-512*sqrt(2)*n)/(15*pi)},
  {9, 1, 10, (-256*sqrt(2)*n*(1 + n^2))/(15*pi)},
  {9, 4, 8,  (-128*sqrt(2)*n*(1 + 4*n^2))/(15*pi)},
  {9, 8, 4,  (512*sqrt(2)*n)/(15*pi)},
  {9, 10, 1, (64*sqrt(2)*n)/(15*pi)},
  {10, 1, 9, (256*sqrt(2)*n*(1 + n^2))/(15*pi)},
  {10, 4, 7, (128*sqrt(2)*n*(1 + 4*n^2))/(15*pi)},
  {10, 7, 4, (-512*sqrt(2)*n)/(15*pi)},
  {10, 9, 1, (-64*sqrt(2)*n)/(15*pi)},
})
--- `c_{i,j} = (F_i, \partial_x F_j)_a`.
atmos.c = ctt({
  {2, 3,      n},
  {3, 2,      -n},
  {5, 6,      n},
  {6, 5,      -n},
  {7, 8,      2*n},
  {8, 7,      -2*n},
  {9, 10,     2*n},
  {10, 9,     -2*n},
})
--- `g_{i,j,k} = (F_i, J(F_j, F_k))_a`.
atmos.g = ctt({
  {1, 2, 3,  (-8*sqrt(2)*n)/(3*pi)},
  {1, 3, 2,  (8*sqrt(2)*n)/(3*pi)},
  {1, 5, 6,  (-32*sqrt(2)*n)/(15*pi)},
  {1, 6, 5,  (32*sqrt(2)*n)/(15*pi)},
  {1, 7, 8,  (-16*sqrt(2)*n)/(3*pi)},
  {1, 8, 7,  (16*sqrt(2)*n)/(3*pi)},
  {1, 9, 10, (-64*sqrt(2)*n)/(15*pi)},
  {1, 10, 9, (64*sqrt(2)*n)/(15*pi)},
  {2, 1, 3,  (8*sqrt(2)*n)/(3*pi)},
  {2, 3, 1,  (-8*sqrt(2)*n)/(3*pi)},
  {2, 4, 6,  (64*sqrt(2)*n)/(15*pi)},
  {2, 5, 8,  (3*n)/2},
  {2, 6, 4,  (-64*sqrt(2)*n)/(15*pi)},
  {2, 6, 7,  (-3*n)/2},
  {2, 7, 6,  (3*n)/2},
  {2, 8, 5,  (-3*n)/2},
  {3, 1, 2,  (-8*sqrt(2)*n)/(3*pi)},
  {3, 2, 1,  (8*sqrt(2)*n)/(3*pi)},
  {3, 4, 5,  (-64*sqrt(2)*n)/(15*pi)},
  {3, 5, 4,  (64*sqrt(2)*n)/(15*pi)},
  {3, 5, 7,  (-3*n)/2},
  {3, 6, 8,  (-3*n)/2},
  {3, 7, 5,  (3*n)/2},
  {3, 8, 6,  (3*n)/2},
  {4, 2, 6,  (-64*sqrt(2)*n)/(15*pi)},
  {4, 3, 5,  (64*sqrt(2)*n)/(15*pi)},
  {4, 5, 3,  (-64*sqrt(2)*n)/(15*pi)},
  {4, 6, 2,  (64*sqrt(2)*n)/(15*pi)},
  {4, 7, 10, (-128*sqrt(2)*n)/(15*pi)},
  {4, 8, 9,  (128*sqrt(2)*n)/(15*pi)},
  {4, 9, 8,  (-128*sqrt(2)*n)/(15*pi)},
  {4, 10, 7, (128*sqrt(2)*n)/(15*pi)},
  {5, 1, 6,  (32*sqrt(2)*n)/(15*pi)},
  {5, 2, 8,  (-3*n)/2},
  {5, 3, 4,  (-64*sqrt(2)*n)/(15*pi)},
  {5, 3, 7,  (3*n)/2},
  {5, 4, 3,  (64*sqrt(2)*n)/(15*pi)},
  {5, 6, 1,  (-32*sqrt(2)*n)/(15*pi)},
  {5, 7, 3,  (-3*n)/2},
  {5, 8, 2,  (3*n)/2},
  {6, 1, 5,  (-32*sqrt(2)*n)/(15*pi)},
  {6, 2, 4,  (64*sqrt(2)*n)/(15*pi)},
  {6, 2, 7,  (3*n)/2},
  {6, 3, 8,  (3*n)/2},
  {6, 4, 2,  (-64*sqrt(2)*n)/(15*pi)},
  {6, 5, 1,  (32*sqrt(2)*n)/(15*pi)},
  {6, 7, 2,  (-3*n)/2},
  {6, 8, 3,  (-3*n)/2},
  {7, 1, 8,  (16*sqrt(2)*n)/(3*pi)},
  {7, 2, 6,  (-3*n)/2},
  {7, 3, 5,  (-3*n)/2},
  {7, 4, 10, (128*sqrt(2)*n)/(15*pi)},
  {7, 5, 3,  (3*n)/2},
  {7, 6, 2,  (3*n)/2},
  {7, 8, 1,  (-16*sqrt(2)*n)/(3*pi)},
  {7, 10, 4, (-128*sqrt(2)*n)/(15*pi)},
  {8, 1, 7,  (-16*sqrt(2)*n)/(3*pi)},
  {8, 2, 5,  (3*n)/2},
  {8, 3, 6,  (-3*n)/2},
  {8, 4, 9,  (-128*sqrt(2)*n)/(15*pi)},
  {8, 5, 2,  (-3*n)/2},
  {8, 6, 3,  (3*n)/2},
  {8, 7, 1,  (16*sqrt(2)*n)/(3*pi)},
  {8, 9, 4,  (128*sqrt(2)*n)/(15*pi)},
  {9, 1, 10, (64*sqrt(2)*n)/(15*pi)},
  {9, 4, 8,  (128*sqrt(2)*n)/(15*pi)},
  {9, 8, 4,  (-128*sqrt(2)*n)/(15*pi)},
  {9, 10, 1, (-64*sqrt(2)*n)/(15*pi)},
  {10, 1, 9, (-64*sqrt(2)*n)/(15*pi)},
  {10, 4, 7, (-128*sqrt(2)*n)/(15*pi)},
  {10, 7, 4, (128*sqrt(2)*n)/(15*pi)},
  {10, 9, 1, (64*sqrt(2)*n)/(15*pi)},
})
--- `d_{i,j} = (F_i, \nabla^2 \phi_j)_a`.
-- Forcing of the ocean on the atmosphere.
atmos.d = ctt({
  {1, 2,      (-2*sqrt(2)*(1 + exp((-2*pi*alpha)/n))*n^2*(n^2 + 4*(4 + alpha^2)))/(3.*pi^2*(n^2 + 4*alpha^2))},
  {1, 4,      (-4*sqrt(2)*(1 - exp((-2*pi*alpha)/n))*n^2*(4 + n^2 + alpha^2))/(3.*pi^2*(n^2 + alpha^2))},
  {2, 1,      ((1 + exp((-2*pi*alpha)/n))*n^2*(3*n^4 + n^2*(12 - 56*alpha^2) - 16*(alpha^2 + alpha^4)))/(2.*pi*(9*n^4 + 40*n^2*alpha^2 + 16*alpha^4))},
  {2, 3,      -(((1 - exp((-2*pi*alpha)/n))*n^2*(1 + 5*n^2 + alpha^2))/(pi*(4*n^2 + alpha^2)))},
  {3, 1,      (-16*(1 + exp((-2*pi*alpha)/n))*n^3*(1 + n^2)*alpha)/(pi*(9*n^4 + 40*n^2*alpha^2 + 16*alpha^4))},
  {3, 3,      (-2*(1 - exp((-2*pi*alpha)/n))*n^3*(1 + n^2))/(pi*(4*n^2*alpha + alpha^3))},
  {4, 1,      (sqrt(2)*(1 + exp((-2*pi*alpha)/n))*n^2*(4 + n^2 + 4*alpha^2))/(3.*pi^2*(n^2 + 4*alpha^2))},
  {4, 3,      (2*sqrt(2)*(1 - exp((-2*pi*alpha)/n))*n^2*(1 + n^2 + alpha^2))/(3.*pi^2*(n^2 + alpha^2))},
  {5, 2,      ((1 + exp((-2*pi*alpha)/n))*n^2*(3*n^4 - 16*alpha^2*(4 + alpha^2)- 8*n^2*(-6 + 7*alpha^2)))/(2.*pi*(9*n^4 + 40*n^2*alpha^2 + 16*alpha^4))},
  {5, 4,      -(((1 - exp((-2*pi*alpha)/n))*n^2*(4 + 5*n^2 + alpha^2))/(pi*(4*n^2 + alpha^2)))},
  {6, 2,      (-16*(1 + exp((-2*pi*alpha)/n))*n^3*(4 + n^2)*alpha)/(pi*(9*n^4 + 40*n^2*alpha^2 + 16*alpha^4))},
  {6, 4,      (-2*(1 - exp((-2*pi*alpha)/n))*n^3*(4 + n^2))/(pi*(4*n^2*alpha + alpha^3))},
  {7, 1,      ((1 + exp((-2*pi*alpha)/n))*n^2*(15*n^4 + n^2*(60 - 200*alpha^2) - 16*(alpha^2 + alpha^4)))/(2.*pi*(225*n^4 + 136*n^2*alpha^2 + 16*alpha^4))},
  {7, 3,      ((1 - exp((-2*pi*alpha)/n))*n^2*(3*n^4 + n^2*(3 - 14*alpha^2) - alpha^2*(1 + alpha^2)))/(pi*(n^2 + alpha^2)*(9*n^2 + alpha^2))},
  {8, 1,      (-32*(1 + exp((-2*pi*alpha)/n))*n^3*(1 + 4*n^2)*alpha)/(pi*(225*n^4 + 136*n^2*alpha^2 + 16*alpha^4))},
  {8, 3,      (-4*(1 - exp((-2*pi*alpha)/n))*n^3*(1 + 4*n^2)*alpha)/(pi*(n^2 + alpha^2)*(9*n^2 + alpha^2))},
  {9, 2,      ((1 + exp((-2*pi*alpha)/n))*n^2*(15*n^4 - 16*alpha^2*(4 + alpha^2) - 40*n^2*(-6 + 5*alpha^2)))/(2.*pi*(225*n^4 + 136*n^2*alpha^2 + 16*alpha^4))},
  {9, 4,      ((1 - exp((-2*pi*alpha)/n))*n^2*(3*n^4 + 2*n^2*(6 - 7*alpha^2) - alpha^2*(4 + alpha^2)))/(pi*(n^2 + alpha^2)*(9*n^2 + alpha^2))},
  {10, 2,      (-128*(1 + exp((-2*pi*alpha)/n))*n^3*(1 + n^2)*alpha)/(pi*(225*n^4 + 136*n^2*alpha^2 + 16*alpha^4))},
  {10, 4,      (-16*(1 - exp((-2*pi*alpha)/n))*n^3*(1 + n^2)*alpha)/(pi*(n^2 + alpha^2)*(9*n^2 + alpha^2))},
})

return {atmos=atmos, ocean=ocean}
