# Coupled ocean-atmosphere model OA-QG-WS v2 #

------------------------------------------------------------------------

## About ##

(c) 2013 Lesley De Cruz

See LICENSE.txt for license information.

This software is provided as supplementary material with:

* Vannitsem, S. and De Cruz, L.: A 24-variable low-order coupled
ocean-atmosphere model: OA-QG-WS v2, Geosci. Model Dev. Discuss., 6, 6569-6604,
[doi:10.5194/gmdd-6-6569-2013](http://dx.doi.org/10.5194/gmdd-6-6569-2013),
2013.

**Please cite this article if you use (a part of) this software for a
publication.**

The authors would appreciate it if you could also send a reprint of
your paper to <svn@meteo.be> and <lesley.decruz@meteo.be>.

Consult [our website](http://climdyn.meteo.be) for any (future) additional
resources.

------------------------------------------------------------------------

## Usage ##

Model and integration parameters can be specified in @{params|params.lua}.
Heating-dependent initial conditions for each of the 24 variables are defined
in @{IC|IC.lua}.

To run a simulation, just run:
    luajit aomodel.lua

In Windows:
    luajit.exe aomodel.lua

This computes the @{stat|mean and variance} of each variable.

The LuaJIT source code can be obtained [here](http://luajit.org/download.html).

------------------------------------------------------------------------

## Implementation notes ##

As the system of differential equations is at most bilinear in y[j] (j=1..n), y
being the @{array} of variables, it can be expressed as a @{tensor} contraction
(written using Einstein convention, i.e. indices that occur twice on one side
of an equation are summed over):

    dy  / dt =  T        y   y      (y  == 1)
      i          i,j,k    j   k       0

The tensor T that encodes the differential equations is composed so that:

* T[i][j][k] contains the contribution of dy[i]/dt proportional to y[j]*y[k].
* Furthermore, y[0] is always equal to 1, so that T[i][0][0] is the constant
contribution to var dy[i]/dt.
* T[i][j][0] + T[i][0][j] is the contribution to  dy[i]/dt which is linear in
y[j].

Ideally, the tensor is composed as an upper triangular matrix (in the last two
coordinates).

The tensor for this model is composed in @{aotensor|aotensor.lua} and uses the
inner products defined in @{inprod|inprod.lua}.

------------------------------------------------------------------------

## References ##

* Charney, J. G. and Straus, D. M.: Form-drag instability, multiple equilibria
and propagating planetary waves in baroclinic, orographically forced, planetary
wave systems, J. Atmos. Sci., 37, 1157-1176, 1980.

* Reinhold, B. B. and Pierrehumbert, R. T.: Dynamics of weather regimes:
quasi-stationary waves and blocking, Mon. Weather Rev., 110, 1105-1145, 1982.

* Reinhold, B. B. and Pierrehumbert, R. T.: Corrections to "Dynamics of
weather regimes: quasi-stationary waves and blocking", Mon. Weather Rev., 113,
2055-2056, 1985.

* Pierini, S.: Low-frequency variability, coherence resonance, and phase
selection in a low-order model of the wind-driven ocean circulation, J.  Phys.
Oceanogr., 41, 1585-1604, 2011.

* Vannitsem, S.: Dynamics and predictability of a low-order wind-driven
ocean-atmosphere coupled model, Clim. Dynam., doi:10.1007/s00382-013-1815-8,
2013.

Please see the [article](http://dx.doi.org/10.5194/gmdd-6-6569-2013) for
the full list of references.
